@@ -374,12 +374,13 @@ static char* toolTip1Header[] = {
     "    var SVGViewBox = null;",
     "    var svgns = 'http://www.w3.org/2000/svg';",
     "    var xlinkns = 'http://www.w3.org/1999/xlink';",
+    "    var map = null;",
     "    var toolTip = null;",
     "    var TrueCoords = null;",
     "    var tipBox = null;",
     "    var tipText = null;",
     "    var tipTitle = null;",
-    "    var tipDesc = null;",
+    "    var tipDesc1 = null;",
     "",
     "    var lastElement = null;",
     "    var titleText = '';",
@@ -391,13 +392,14 @@ static char* toolTip1Header[] = {
     "       SVGDocument = evt.target.ownerDocument;",
     "       SVGRoot = SVGDocument.documentElement;",
     "       TrueCoords = SVGRoot.createSVGPoint();",
+    "       map = document.getElementById('canvas')",
     "",
     "       toolTip = SVGDocument.getElementById('ToolTip');",
     "       tipBox = SVGDocument.getElementById('tipbox');",
     "       tipText = SVGDocument.getElementById('tipText');",
     "       tipText.setAttributeNS(null, 'font-family', 'Arial, sans-serif' );",
     "       tipTitle = SVGDocument.getElementById('tipTitle');",
-    "       tipDesc = SVGDocument.getElementById('tipDesc');",
+    "       tipDesc1 = SVGDocument.getElementById('tipDesc1');",
     "       //window.status = (TrueCoords);",
     "",
     "       //create event for object",
@@ -412,8 +414,8 @@ static char* toolTip1Header[] = {
     "       //    mouse position accordingly",
     "       var newScale = SVGRoot.currentScale;",
     "       var translation = SVGRoot.currentTranslate;",
-    "       TrueCoords.x = (evt.clientX - translation.x)/newScale;",
-    "       TrueCoords.y = (evt.clientY - translation.y)/newScale;",
+    "       TrueCoords.x = (evt.pageX - translation.x)/newScale;",
+    "       TrueCoords.y = (evt.pageY - translation.y)/newScale;",
     "    };",
     "",
     "",
@@ -423,85 +425,99 @@ static char* toolTip1Header[] = {
     "    };",
     "",
     "",
-    "    function ShowTooltip( evt )",
-    "    {",
-    "       GetTrueCoords( evt );",
+    "   function ShowTooltip( evt )",
+    "   {",
+    "      GetTrueCoords( evt );",
     "",
-    "       var tipScale = 1/SVGRoot.currentScale;",
-    "       var textWidth = 0;",
-    "       var tspanWidth = 0;",
-    "       var boxHeight = 20;",
+    "      var tipScale = 1/SVGRoot.currentScale;",
+    "      var textWidth = 0;",
+    "      var tspanWidth = 0;",
+    "      var boxHeight = 20;",
     "",
-    "       tipBox.setAttributeNS(null, 'transform', 'scale(' + tipScale + ',' + tipScale + ')' );",
-    "       tipText.setAttributeNS(null, 'transform', 'scale(' + tipScale + ',' + tipScale + ')' );",
+    "      tipBox.setAttributeNS(null, 'transform', 'scale(' + tipScale + ',' + tipScale + ')' );",
+    "      tipText.setAttributeNS(null, 'transform', 'scale(' + tipScale + ',' + tipScale + ')' );",
     "",
-    "       var titleValue = '';",
-    "       var descValue = '';",
-    "       var targetElement = evt.target;",
-    "       if ( lastElement != targetElement )",
-    "       {",
-    "          var targetTitle = targetElement.getElementsByTagNameNS(svgns, 'title').item(0);",
-    "          if ( targetTitle )",
-    "          {",
-    "             // if there is a 'title' element, use its contents for the tooltip title",
-    "             titleValue = targetTitle.firstChild.nodeValue;",
-    "          }",
+    "      var titleValue = '';",
+    "      var desc1Value = '';",
+    "      var targetElement = evt.target;",
+    "      if ( lastElement != targetElement )",
+    "      {",
+    "         var targetTitle = targetElement.getElementsByTagName('tiptitle').item(0);",
+    "         if ( targetTitle )",
+    "         {",
+    "            // if there is a 'tiptitle' element, use its contents for the tooltip tiptitle",
+    "            titleValue = targetTitle.firstChild.nodeValue;",
+    "         }",
     "",
-    "          var targetDesc = targetElement.getElementsByTagNameNS(svgns, 'desc').item(0);",
-    "          if ( targetDesc )",
-    "          {",
-    "             // if there is a 'desc' element, use its contents for the tooltip desc",
-    "             descValue = targetDesc.firstChild.nodeValue;",
+    "         var targetDesc1 = targetElement.getElementsByTagName('desc1').item(0);",
+    "         if ( targetDesc1 )",
+    "         {",
+    "            // if there is a 'desc1' element, use its contents for the tooltip desc1",
+    "            desc1Value = targetDesc1.firstChild.nodeValue;",
     "",
-    "             if ( '' == titleValue )",
-    "             {",
-    "                // if there is no 'title' element, use the contents of the 'desc' element for the tooltip title instead",
-    "                titleValue = descValue;",
-    "                descValue = '';",
-    "             }",
-    "          }",
+    "            if ( '' == titleValue )",
+    "            {",
+    "               // if there is no 'tiptitle' element, use the contents of the 'desc1' element for the tooltip title instead",
+    "               titleValue = desc1Value;",
+    "               desc1Value = '';",
+    "            }",
+    "         } else {",
+    "             // useful for debugging positioning...",
+    "             // desc1Value = 'xPos=' + TrueCoords.x + ' yPos=' + TrueCoords.y;",
+    "         }",
     "",
-    "          // if there is still no 'title' element, use the contents of the 'id' attribute for the tooltip title",
-    "          if ( '' == titleValue )",
-    "          {",
+    "         // if there is still no 'tiptitle' element, use the contents of the 'id' attribute for the tooltip title",
+    "         if ( '' == titleValue)",
+    "         {",
     "             titleValue = targetElement.getAttributeNS(null, 'id');",
-    "          }",
+    "             if (titleValue = 'canvas')",
+    "                 titleValue = ''",
+    "         }",
     "",
-    "          // selectively assign the tooltip title and desc the proper values,",
-    "          //   and hide those which don't have text values",
-    "          //",
-    "          var titleDisplay = 'none';",
-    "          if ( '' != titleValue )",
-    "          {",
-    "             tipTitle.firstChild.nodeValue = titleValue;",
-    "             titleDisplay = 'inline';",
-    "          }",
-    "          tipTitle.setAttributeNS(null, 'display', titleDisplay );",
+    "         // selectively assign the tooltip title and desc the proper values,",
+    "         //   and hide those which don't have text values",
+    "         //",
+    "         var titleDisplay = 'none';",
+    "         if ( '' != titleValue )",
+    "         {",
+    "            tipTitle.firstChild.nodeValue = titleValue;",
+    "            titleDisplay = 'inline';",
+    "         }",
+    "         tipTitle.setAttributeNS(null, 'display', titleDisplay );",
     "",
+    "         var desc1Display = 'none';",
+    "         if ( '' != desc1Value )",
+    "         {",
+    "            tipDesc1.firstChild.nodeValue = desc1Value;",
+    "            desc1Display = 'inline';",
+    "         }",
+    "         tipDesc1.setAttributeNS(null, 'display', desc1Display );",
+    "      }",
     "",
-    "          var descDisplay = 'none';",
-    "          if ( '' != descValue )",
-    "          {",
-    "             tipDesc.firstChild.nodeValue = descValue;",
-    "             descDisplay = 'inline';",
-    "          }",
-    "          tipDesc.setAttributeNS(null, 'display', descDisplay );",
-    "       }",
+    "      // if there are tooltip contents to be displayed, adjust the size and position of the box",
+    "      if ( '' != titleValue )",
+    "      {",
+    "         var xPos = TrueCoords.x;",
+    "         var yPos = TrueCoords.y;",
+    "         //return rectangle around text as SVGRect object",
+    "         var outline = tipText.getBBox();",
+    "         tipBox.setAttributeNS(null, 'width', Number(outline.width) + 10);",
+    "         tipBox.setAttributeNS(null, 'height', Number(outline.height) + 10);",
     "",
-    "       // if there are tooltip contents to be displayed, adjust the size and position of the box",
-    "       if ( '' != titleValue )",
-    "       {",
-    "          var xPos = TrueCoords.x + (10 * tipScale);",
-    "          var yPos = TrueCoords.y + (10 * tipScale);",
+    "         // work out which direction tooltip needs to hang in to be visible",
+    "         var mapbb = map.getBBox();",
+    "         if (xPos + (10 * tipScale) + Number(outline.width) < Number(mapbb.width))",
+    "             xPos = xPos + 10 * tipScale;",
+    "         else",
+    "             xPos = xPos - 10 * tipScale - Number(outline.width);",
+    "         if (yPos + (10 * tipScale) + Number(outline.height) < Number(mapbb.height))",
+    "             yPos = yPos + 10 * tipScale;",
+    "         else",
+    "             yPos = yPos - 10 * tipScale - Number(outline.height);",
     "",
-    "          //return rectangle around text as SVGRect object",
-    "          var outline = tipText.getBBox();",
-    "          tipBox.setAttributeNS(null, 'width', Number(outline.width) + 10);",
-    "          tipBox.setAttributeNS(null, 'height', Number(outline.height) + 10);",
-    "",
-    "          // update position",
-    "          toolTip.setAttributeNS(null, 'transform', 'translate(' + xPos + ',' + yPos + ')');",
-    "          toolTip.setAttributeNS(null, 'visibility', 'visible');",
+    "         // update position",
+    "         toolTip.setAttributeNS(null, 'transform', 'translate(' + xPos + ',' + yPos + ')');",
+    "         toolTip.setAttributeNS(null, 'visibility', 'visible');",
     "       }",
     "    };",
     "",
@@ -521,6 +537,7 @@ static char* toolTip2Header[] = {
     "   var SVGViewBox = null;",
     "   var svgns = 'http://www.w3.org/2000/svg';",
     "   var xlinkns = 'http://www.w3.org/1999/xlink';",
+    "   var map = null;",
     "   var toolTip = null;",
     "   var TrueCoords = null;",
     "   var tipBox = null;",
@@ -539,6 +556,7 @@ static char* toolTip2Header[] = {
     "      SVGDocument = evt.target.ownerDocument;",
     "      SVGRoot = SVGDocument.documentElement;",
     "      TrueCoords = SVGRoot.createSVGPoint();",
+    "      map = document.getElementById('canvas')",
     "",
     "      toolTip = SVGDocument.getElementById('ToolTip');",
     "      tipBox = SVGDocument.getElementById('tipbox');",
@@ -561,8 +579,8 @@ static char* toolTip2Header[] = {
     "      //    mouse position accordingly",
     "      var newScale = SVGRoot.currentScale;",
     "      var translation = SVGRoot.currentTranslate;",
-    "      TrueCoords.x = (evt.clientX - translation.x)/newScale;",
-    "      TrueCoords.y = (evt.clientY - translation.y)/newScale;",
+    "      TrueCoords.x = (evt.pageX - translation.x)/newScale;",
+    "      TrueCoords.y = (evt.pageY - translation.y)/newScale;",
     "   };",
     "",
     "",
@@ -590,88 +608,312 @@ static char* toolTip2Header[] = {
     "      var targetElement = evt.target;",
     "      if ( lastElement != targetElement )",
     "      {",
-    "	 var targetTitle = targetElement.getElementsByTagNameNS(svgns, 'title').item(0);",
-    "	 if ( targetTitle )",
-    "	 {",
-    "	    // if there is a 'title' element, use its contents for the tooltip title",
-    "	    titleValue = targetTitle.firstChild.nodeValue;",
-    "	 }",
+    "         var targetTitle = targetElement.getElementsByTagName('tiptitle').item(0);",
+    "         if ( targetTitle )",
+    "         {",
+    "            // if there is a 'tiptitle' element, use its contents for the tooltip tiptitle",
+    "            titleValue = targetTitle.firstChild.nodeValue;",
+    "         }",
     "",
-    "	 var targetDesc1 = targetElement.getElementsByTagNameNS(svgns, 'desc1').item(0);",
-    "	 if ( targetDesc1 )",
-    "	 {",
-    "	    // if there is a 'desc1' element, use its contents for the tooltip desc1",
-    "	    desc1Value = targetDesc1.firstChild.nodeValue;",
+    "         var targetDesc1 = targetElement.getElementsByTagName('desc1').item(0);",
+    "         if ( targetDesc1 )",
+    "         {",
+    "            // if there is a 'desc1' element, use its contents for the tooltip desc1",
+    "            desc1Value = targetDesc1.firstChild.nodeValue;",
     "",
-    "	    if ( '' == titleValue )",
-    "	    {",
-    "	       // if there is no 'title' element, use the contents of the 'desc1' element for the tooltip title instead",
-    "	       titleValue = desc1Value;",
-    "	       desc1Value = '';",
-    "	    }",
-    "	 }",
-    "	 var targetDesc2 = targetElement.getElementsByTagNameNS(svgns, 'desc2').item(0);",
-    "	 if ( targetDesc2 )",
-    "	 {",
-    "	    // if there is a 'desc2' element, use its contents for the tooltip desc",
-    "	    desc2Value = targetDesc2.firstChild.nodeValue;",
-    "	 }",
+    "            if ( '' == titleValue )",
+    "            {",
+    "               // if there is no 'tiptitle' element, use the contents of the 'desc1' element for the tooltip title instead",
+    "               titleValue = desc1Value;",
+    "               desc1Value = '';",
+    "            }",
+    "         } else {",
+    "             // useful for debugging positioning...",
+    "             // desc1Value = 'xPos=' + TrueCoords.x + ' yPos=' + TrueCoords.y;",
+    "         }",
+    "         var targetDesc2 = targetElement.getElementsByTagName('desc2').item(0);",
+    "         if ( targetDesc2 )",
+    "         {",
+    "             // if there is a 'desc2' element, use its contents for the tooltip desc",
+    "             desc2Value = targetDesc2.firstChild.nodeValue;",
+    "         } else {",
+    "             // useful for debugging positioning... here bounding box of root element",
+    "             // desc2Value = 'bbox=' + map.getBBox().width + ' x ' + map.getBBox().height;",
+    "         }",
     "",
-    "	 // if there is still no 'title' element, use the contents of the 'id' attribute for the tooltip title",
-    "	 if ( '' == titleValue )",
-    "	 {",
-    "	    titleValue = targetElement.getAttributeNS(null, 'id');",
-    "	 }",
+    "         // if there is still no 'tiptitle' element, use the contents of the 'id' attribute for the tooltip title",
+    "         if ( '' == titleValue)",
+    "         {",
+    "             titleValue = targetElement.getAttributeNS(null, 'id');",
+    "             if (titleValue = 'canvas')",
+    "                 titleValue = ''",
+    "         }",
     "",
-    "	 // selectively assign the tooltip title and desc the proper values,",
-    "	 //   and hide those which don't have text values",
-    "	 //",
-    "	 var titleDisplay = 'none';",
-    "	 if ( '' != titleValue )",
-    "	 {",
-    "	    tipTitle.firstChild.nodeValue = titleValue;",
-    "	    titleDisplay = 'inline';",
-    "	 }",
-    "	 tipTitle.setAttributeNS(null, 'display', titleDisplay );",
+    "         // selectively assign the tooltip title and desc the proper values,",
+    "         //   and hide those which don't have text values",
+    "         //",
+    "         var titleDisplay = 'none';",
+    "         if ( '' != titleValue )",
+    "         {",
+    "            tipTitle.firstChild.nodeValue = titleValue;",
+    "            titleDisplay = 'inline';",
+    "         }",
+    "         tipTitle.setAttributeNS(null, 'display', titleDisplay );",
     "",
-    "	 var desc1Display = 'none';",
-    "	 if ( '' != desc1Value )",
-    "	 {",
-    "	    tipDesc1.firstChild.nodeValue = desc1Value;",
-    "	    desc1Display = 'inline';",
-    "	 }",
-    "	 tipDesc1.setAttributeNS(null, 'display', desc1Display );",
+    "         var desc1Display = 'none';",
+    "         if ( '' != desc1Value )",
+    "         {",
+    "            tipDesc1.firstChild.nodeValue = desc1Value;",
+    "            desc1Display = 'inline';",
+    "         }",
+    "         tipDesc1.setAttributeNS(null, 'display', desc1Display );",
     "",
-    "	 var desc2Display = 'none';",
-    "	 if ( '' != desc2Value )",
-    "	 {",
-    "	    tipDesc2.firstChild.nodeValue = desc2Value;",
-    "	    desc2Display = 'inline';",
-    "	 }",
-    "	 tipDesc2.setAttributeNS(null, 'display', desc2Display );",
+    "         var desc2Display = 'none';",
+    "         if ( '' != desc2Value )",
+    "         {",
+    "            tipDesc2.firstChild.nodeValue = desc2Value;",
+    "            desc2Display = 'inline';",
+    "         }",
+    "         tipDesc2.setAttributeNS(null, 'display', desc2Display );",
     "      }",
     "",
     "      // if there are tooltip contents to be displayed, adjust the size and position of the box",
     "      if ( '' != titleValue )",
     "      {",
-    "	 var xPos = TrueCoords.x + (10 * tipScale);",
-    "	 var yPos = TrueCoords.y + (10 * tipScale);",
+    "         var xPos = TrueCoords.x;",
+    "         var yPos = TrueCoords.y;",
+    "         //return rectangle around text as SVGRect object",
+    "         var outline = tipText.getBBox();",
+    "         tipBox.setAttributeNS(null, 'width', Number(outline.width) + 10);",
+    "         tipBox.setAttributeNS(null, 'height', Number(outline.height) + 10);",
     "",
-    "	 //return rectangle around text as SVGRect object",
-    "	 var outline = tipText.getBBox();",
-    "	 tipBox.setAttributeNS(null, 'width', Number(outline.width) + 10);",
-    "	 tipBox.setAttributeNS(null, 'height', Number(outline.height) + 10);",
+    "         // work out which direction tooltip needs to hang in to be visible",
+    "         var mapbb = map.getBBox();",
+    "         if (xPos + (10 * tipScale) + Number(outline.width) < Number(mapbb.width))",
+    "             xPos = xPos + 10 * tipScale;",
+    "         else",
+    "             xPos = xPos - 10 * tipScale - Number(outline.width);",
+    "         if (yPos + (10 * tipScale) + Number(outline.height) < Number(mapbb.height))",
+    "             yPos = yPos + 10 * tipScale;",
+    "         else",
+    "             yPos = yPos - 10 * tipScale - Number(outline.height);",
     "",
-    "	 // update position",
-    "	 toolTip.setAttributeNS(null, 'transform', 'translate(' + xPos + ',' + yPos + ')');",
-    "	 toolTip.setAttributeNS(null, 'visibility', 'visible');",
-    "      }",
-    "   };",
+    "         // update position",
+    "         toolTip.setAttributeNS(null, 'transform', 'translate(' + xPos + ',' + yPos + ')');",
+    "         toolTip.setAttributeNS(null, 'visibility', 'visible');",
+    "       }",
+    "    };",
     "",
     "]]></script>",
     "!END!"
 };
 
+/*
+static char* toolTip3Header[] = {
+    "<script type=\"text/ecmascript\"><![CDATA[",
+    "   // This code for a title + two-line tooltip mode displays OK in",
+    "   // the default SVG renderers in Firefox and Safari, but does not",
+    "   // display in the Batik standalone SVG viewer. It probably needs",
+    "   // some declarations to extend XML elements to make it fully",
+    "   // compliant with the SVG specs, see man/RSVGTipsDevice.future.Rd.",
+    "   var SVGDocument = null;",
+    "   var SVGRoot = null;",
+    "   var SVGViewBox = null;",
+    "   var svgns = 'http://www.w3.org/2000/svg';",
+    "   var xlinkns = 'http://www.w3.org/1999/xlink';",
+    "   var map = null;",
+    "   var toolTip = null;",
+    "   var TrueCoords = null;",
+    "   var tipBox = null;",
+    "   var tipText = null;",
+    "   var tipTitle = null;",
+    "   var tipDesc1 = null;",
+    "   var tipDesc2 = null;",
+    "   var tipDesc3 = null;",
+    "",
+    "   var lastElement = null;",
+    "   var titleText = '';",
+    "   var titleDesc = '';",
+    "",
+    "",
+    "   function Init(evt)",
+    "   {",
+    "      SVGDocument = evt.target.ownerDocument;",
+    "      SVGRoot = SVGDocument.documentElement;",
+    "      TrueCoords = SVGRoot.createSVGPoint();",
+    "      map = document.getElementById('canvas')",
+    "",
+    "      toolTip = SVGDocument.getElementById('ToolTip');",
+    "      tipBox = SVGDocument.getElementById('tipbox');",
+    "      tipText = SVGDocument.getElementById('tipText');",
+    "      tipText.setAttributeNS(null, 'font-family', 'Arial, sans-serif' );",
+    "      tipTitle = SVGDocument.getElementById('tipTitle');",
+    "      tipDesc1 = SVGDocument.getElementById('tipDesc1');",
+    "      tipDesc2 = SVGDocument.getElementById('tipDesc2');",
+    "      tipDesc3 = SVGDocument.getElementById('tipDesc3');",
+    "      //window.status = (TrueCoords);",
+    "",
+    "      //create event for object",
+    "      SVGRoot.addEventListener('mousemove', ShowTooltip, false);",
+    "      SVGRoot.addEventListener('mouseout', HideTooltip, false);",
+    "   };",
+    "",
+    "",
+    "   function GetTrueCoords(evt)",
+    "   {",
+    "      // find the current zoom level and pan setting, and adjust the reported",
+    "      //    mouse position accordingly",
+    "      var newScale = SVGRoot.currentScale;",
+    "      var translation = SVGRoot.currentTranslate;",
+    "      TrueCoords.x = (evt.pageX - translation.x)/newScale;",
+    "      TrueCoords.y = (evt.pageY - translation.y)/newScale;",
+    "   };",
+    "",
+    "",
+    "   function HideTooltip( evt )",
+    "   {",
+    "      toolTip.setAttributeNS(null, 'visibility', 'hidden');",
+    "   };",
+    "",
+    "",
+    "   function ShowTooltip( evt )",
+    "   {",
+    "      GetTrueCoords( evt );",
+    "",
+    "      var tipScale = 1/SVGRoot.currentScale;",
+    "      var textWidth = 0;",
+    "      var tspanWidth = 0;",
+    "      var boxHeight = 20;",
+    "",
+    "      tipBox.setAttributeNS(null, 'transform', 'scale(' + tipScale + ',' + tipScale + ')' );",
+    "      tipText.setAttributeNS(null, 'transform', 'scale(' + tipScale + ',' + tipScale + ')' );",
+    "",
+    "      var titleValue = '';",
+    "      var desc1Value = '';",
+    "      var desc2Value = '';",
+    "      var desc3Value = '';",
+    "      var targetElement = evt.target;",
+    "      if ( lastElement != targetElement )",
+    "      {",
+    "         var targetTitle = targetElement.getElementsByTagName('tiptitle').item(0);",
+    "         if ( targetTitle )",
+    "         {",
+    "            // if there is a 'tiptitle' element, use its contents for the tooltip tiptitle",
+    "            titleValue = targetTitle.firstChild.nodeValue;",
+    "         }",
+    "",
+    "         var targetDesc1 = targetElement.getElementsByTagName('desc1').item(0);",
+    "         if ( targetDesc1 )",
+    "         {",
+    "            // if there is a 'desc1' element, use its contents for the tooltip desc1",
+    "            desc1Value = targetDesc1.firstChild.nodeValue;",
+    "",
+    "            if ( '' == titleValue )",
+    "            {",
+    "               // if there is no 'tiptitle' element, use the contents of the 'desc1' element for the tooltip title instead",
+    "               titleValue = desc1Value;",
+    "               desc1Value = '';",
+    "            }",
+    "         } else {",
+    "             // useful for debugging positioning...",
+    "             // desc1Value = 'xPos=' + TrueCoords.x + ' yPos=' + TrueCoords.y;",
+    "         }",
+    "         var targetDesc2 = targetElement.getElementsByTagName('desc2').item(0);",
+    "         if ( targetDesc2 )",
+    "         {",
+    "             // if there is a 'desc2' element, use its contents for the tooltip desc",
+    "             desc2Value = targetDesc2.firstChild.nodeValue;",
+    "         } else {",
+    "             // useful for debugging positioning... here bounding box of root element",
+    "             // desc2Value = 'bbox=' + map.getBBox().width + ' x ' + map.getBBox().height;",
+    "         }",
+    "",
+    "         var targetDesc3 = targetElement.getElementsByTagName('desc3').item(0);",
+    "         if ( targetDesc3 )",
+    "         {",
+    "             // if there is a 'desc3' element, use its contents for the tooltip desc",
+    "             desc3Value = targetDesc3.firstChild.nodeValue;",
+    "         } else {",
+    "             // useful for debugging positioning... here bounding box of root element",
+    "             // desc3Value = 'bbox=' + map.getBBox().width + ' x ' + map.getBBox().height;",
+    "         }",
+    "",
+    "         // if there is still no 'tiptitle' element, use the contents of the 'id' attribute for the tooltip title",
+    "         if ( '' == titleValue)",
+    "         {",
+    "             titleValue = targetElement.getAttributeNS(null, 'id');",
+    "             if (titleValue = 'canvas')",
+    "                 titleValue = ''",
+    "         }",
+    "",
+    "         // selectively assign the tooltip title and desc the proper values,",
+    "         //   and hide those which don't have text values",
+    "         //",
+    "         var titleDisplay = 'none';",
+    "         if ( '' != titleValue )",
+    "         {",
+    "            tipTitle.firstChild.nodeValue = titleValue;",
+    "            titleDisplay = 'inline';",
+    "         }",
+    "         tipTitle.setAttributeNS(null, 'display', titleDisplay );",
+    "",
+    "         var desc1Display = 'none';",
+    "         if ( '' != desc1Value )",
+    "         {",
+    "            tipDesc1.firstChild.nodeValue = desc1Value;",
+    "            desc1Display = 'inline';",
+    "         }",
+    "         tipDesc1.setAttributeNS(null, 'display', desc1Display );",
+    "",
+    "         var desc2Display = 'none';",
+    "         if ( '' != desc2Value )",
+    "         {",
+    "            tipDesc2.firstChild.nodeValue = desc2Value;",
+    "            desc2Display = 'inline';",
+    "         }",
+    "         tipDesc2.setAttributeNS(null, 'display', desc2Display );",
+    "",
+    "         var desc3Display = 'none';",
+    "         if ( '' != desc3Value )",
+    "         {",
+    "            tipDesc3.firstChild.nodeValue = desc3Value;",
+    "            desc3Display = 'inline';",
+    "         }",
+    "         tipDesc3.setAttributeNS(null, 'display', desc3Display );",
+    "      }",
+    "",
+    "      // if there are tooltip contents to be displayed, adjust the size and position of the box",
+    "      if ( '' != titleValue )",
+    "      {",
+    "         var xPos = TrueCoords.x;",
+    "         var yPos = TrueCoords.y;",
+    "         //return rectangle around text as SVGRect object",
+    "         var outline = tipText.getBBox();",
+    "         tipBox.setAttributeNS(null, 'width', Number(outline.width) + 10);",
+    "         tipBox.setAttributeNS(null, 'height', Number(outline.height) + 10);",
+    "",
+    "         // work out which direction tooltip needs to hang in to be visible",
+    "         var mapbb = map.getBBox();",
+    "         if (xPos + (10 * tipScale) + Number(outline.width) < Number(mapbb.width))",
+    "             xPos = xPos + 10 * tipScale;",
+    "         else",
+    "             xPos = xPos - 10 * tipScale - Number(outline.width);",
+    "         if (yPos + (10 * tipScale) + Number(outline.height) < Number(mapbb.height))",
+    "             yPos = yPos + 10 * tipScale;",
+    "         else",
+    "             yPos = yPos - 10 * tipScale - Number(outline.height);",
+    "",
+    "         // update position",
+    "         toolTip.setAttributeNS(null, 'transform', 'translate(' + xPos + ',' + yPos + ')');",
+    "         toolTip.setAttributeNS(null, 'visibility', 'visible');",
+    "       }",
+    "    };",
+    "",
+    "]]></script>",
+    "!END!"
+};
+*/
+
 /* Device driver actions */
 
 static void   SVG_Activate(pDevDesc dd);
@@ -996,7 +1238,7 @@ static void SVG_header(SVGDesc *ptd)
     fprintf(ptd->texfp, "     xmlns:ev=\"http://www.w3.org/2001/xml-events\"\n");
     fprintf(ptd->texfp, "     width=\"%.2f\" height=\"%.2f\"\n",
 	    in2dots(ptd->width), in2dots(ptd->height));
-    fprintf(ptd->texfp, "     viewBox=\"0,0,%.2f,%.2f\"\n",
+    fprintf(ptd->texfp, "     viewBox=\"0 0 %.2f %.2f\"\n",
 	    in2dots(ptd->width), in2dots(ptd->height));
     fprintf(ptd->texfp, "     font-family=\"Arial, Helvetica, sans-serif\"\n");
     if (ptd->toolTipMode)
@@ -1015,8 +1257,9 @@ static void SVG_header(SVGDesc *ptd)
     else if (ptd->toolTipMode==2)
 	for (i=0; strcmp("!END!", toolTip2Header[i]); i++)
 	    fprintf(ptd->texfp, "%s\n", toolTip2Header[i]);
+    /* This element named 'canvas' is what allows map.getBBox() to work */
     fprintf(ptd->texfp,
-	    "<rect width=\"100%%\" height=\"100%%\" style=\"fill:%s\"/>\n",
+	    "<rect width=\"100%%\" height=\"100%%\" style=\"fill:%s\" id='canvas'/>\n",
 	    col2RGBname(ptd->bg));
 
     ptd->pageno++;
@@ -1033,13 +1276,20 @@ static void SVG_footer(SVGDesc *ptd)
 		2 * ptd->tipFontSize, ptd->tipFontSize);
 	fprintf(ptd->texfp, "      <tspan id='tipTitle' x='5' font-weight='bold' fill='black'><![CDATA[]]></tspan>\n");
 	if (ptd->toolTipMode==1) {
-	    fprintf(ptd->texfp, "      <tspan id='tipDesc' x='5' dy='%d'><![CDATA[]]></tspan>\n",
+	    fprintf(ptd->texfp, "      <tspan id='tipDesc1' x='5' dy='%d'><![CDATA[]]></tspan>\n",
 		    (int) ((1.3 * ptd->tipFontSize) + 0.5));
-	} else {
+	} else if (ptd->toolTipMode==2) {
 	    fprintf(ptd->texfp, "      <tspan id='tipDesc1' x='5' dy='%d'><![CDATA[]]></tspan>\n",
 		    (int) ((1.3 * ptd->tipFontSize) + 0.5));
 	    fprintf(ptd->texfp, "      <tspan id='tipDesc2' x='5' dy='%d'><![CDATA[]]></tspan>\n",
 		    (int) ((1.3 * ptd->tipFontSize) + 0.5));
+	} else if (ptd->toolTipMode==3) {
+	    fprintf(ptd->texfp, "      <tspan id='tipDesc1' x='5' dy='%d'><![CDATA[]]></tspan>\n",
+		    (int) ((1.3 * ptd->tipFontSize) + 0.5));
+	    fprintf(ptd->texfp, "      <tspan id='tipDesc2' x='5' dy='%d'><![CDATA[]]></tspan>\n",
+		    (int) ((1.3 * ptd->tipFontSize) + 0.5));
+	    fprintf(ptd->texfp, "      <tspan id='tipDesc3' x='5' dy='%d'><![CDATA[]]></tspan>\n",
+		    (int) ((1.3 * ptd->tipFontSize) + 0.5));
 	}
 	fprintf(ptd->texfp, "   </text>\n");
 	fprintf(ptd->texfp, "</g>\n");
@@ -1099,7 +1349,7 @@ static void SVG_Clip(double x0, double x1, double y0, double y1,
 static void SVG_NewPage(const pGEcontext gc, pDevDesc dd)
 {
     SVGDesc *ptd = (SVGDesc *) dd->deviceSpecific;
-    int face, size;
+    int size;
 
     if (ptd->onefile){
     } else if (ptd->pageno) {
@@ -1107,7 +1357,7 @@ static void SVG_NewPage(const pGEcontext gc, pDevDesc dd)
 	SVG_header(ptd);
     } else ptd->pageno++;
 
-    face = ptd->fontface;
+    /* face = ptd->fontface; */
     size = ptd->fontsize;
     ptd->fontface = 0;
     ptd->fontsize = 0;
@@ -1463,6 +1713,7 @@ Rboolean SVGDeviceDriver(pDevDesc dd, char *filename, char *bg, char *fg,
     ptd->shapeContents = 0;
     ptd->shapeContentsUsed = 0;
     ptd->shapeURL = 0;
+    ptd->shapeURLTarget = 0;
     ptd->shapeURLUsed = 0;
     ptd->toolTipMode = ((toolTipMode>0 && toolTipMode<=2) ? toolTipMode : 0);
     ptd->title = 0;
